package com.hive.sample

import android.app.Activity
import android.os.Handler
import android.os.Looper
import android.view.View
import android.widget.EditText
import android.widget.TextView
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.DataStore
import com.hive.ResultAPI
import com.hive.base.Property
import com.hive.sample.uihelper.ViewPagerBase


/**
 * HIVE SDK 의 DataStore 기능 테스트 뷰
 *
 * @author hife
 */
class DataStoreTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {

    private val dataStoreProperty: Property by lazy { Property("datastore_property") }

    private val key0: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_key0) as EditText }
    private val key1: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_key1) as EditText }
    private val key2: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_key2) as EditText }
    private val key3: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_key3) as EditText }
    private val value0: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_value0) as EditText }
    private val value1: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_value1) as EditText }
    private val value2: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_value2) as EditText }
    private val value3: EditText by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.data_store_value3) as EditText }
    private val tvResponse: TextView by lazy { this@DataStoreTestView.view.findViewById<View>(R.id.tvDataStoreResponseData) as TextView }


    init {
        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)

        this.view.findViewById<View>(R.id.btnDataStoreSetKey).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnDataStoreSetData).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnDataStoreGet).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnDataStoreGetMyData).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnDataStoreGetUsersData).setOnClickListener(this)

        with(dataStoreProperty) {
            getValue("datastore.key0")?.let { if (it.isNotBlank()) key0.setText(it) }
            getValue("datastore.key1")?.let { if (it.isNotBlank()) key1.setText(it) }
            getValue("datastore.key2")?.let { if (it.isNotBlank()) key2.setText(it) }
            getValue("datastore.key3")?.let { if (it.isNotBlank()) key3.setText(it) }
            getValue("datastore.value0")?.let { if (it.isNotBlank()) value0.setText(it) }
            getValue("datastore.value1")?.let { if (it.isNotBlank()) value1.setText(it) }
            getValue("datastore.value2")?.let { if (it.isNotBlank()) value2.setText(it) }
            getValue("datastore.value3")?.let { if (it.isNotBlank()) value3.setText(it) }
        }
    }

    private fun saveText() {
        with(dataStoreProperty) {
            setValue("datastore.key0", key0.text.toString())
            setValue("datastore.key1", key1.text.toString())
            setValue("datastore.key2", key2.text.toString())
            setValue("datastore.key3", key3.text.toString())
            setValue("datastore.value0", value0.text.toString())
            setValue("datastore.value1", value1.text.toString())
            setValue("datastore.value2", value2.text.toString())
            setValue("datastore.value3", value3.text.toString())
            writeProperties()
        }
    }


    override fun onClick(v: View) {

        Handler(Looper.getMainLooper()).post {
            tvResponse.text = ""
        }

        saveText()

        when (v.id) {


            R.id.btnDataStoreSetKey -> {
                addLog("\n\n=== DataStore 데이터 설정 (key 0) ===\n")
                addLog("DataStore.set(key0, value0, listener) Called\n")

                DataStore.set(key0.text.toString(), value0.text.toString(), object : DataStore.DataStoreSetListener {
                    override fun onDataStoreSet(result: ResultAPI) {
                        val msg = "DataStore.set(key 0) Callback\n" +
                                "\n" +
                                "$result"
                        addLog(msg)

                        Handler(Looper.getMainLooper()).post {
                            tvResponse.text = msg
                        }
                    }
                })
            }


            R.id.btnDataStoreSetData -> {
                addLog("\n\n=== DataStore 데이터 설정 (key 1 ~ 3) ===\n")
                addLog("DataStore.set(key 1 ~ 3, listener) Called\n")

                val map = mapOf(
                    key1.text.toString() to value1.text.toString(),
                    key2.text.toString() to value2.text.toString(),
                    key3.text.toString() to value3.text.toString(),
                )

                DataStore.set(map, object : DataStore.DataStoreSetListener {
                    override fun onDataStoreSet(result: ResultAPI) {
                        val msg = "DataStore.set(key 1 ~ 3) Callback\n" +
                                "\n" +
                                "$result"
                        addLog(msg)

                        Handler(Looper.getMainLooper()).post {
                            tvResponse.text = msg
                        }
                    }
                })
            }


            R.id.btnDataStoreGet -> {
                addLog("\n\n=== DataStore 내 데이터 하나 가져오기 (key 0) ===\n")
                addLog("DataStore.get(key0, listener) Called\n")

                DataStore.get(key0.text.toString(), object : DataStore.DataStoreGetListener {
                    override fun onDataStoreGet(result: ResultAPI, data: String?) {
                        val msg = "DataStore.get(key 0) Callback\n" +
                                "\n" +
                                "$result\n" +
                                "\n" +
                                "$data"
                        addLog(msg)

                        Handler(Looper.getMainLooper()).post {
                            tvResponse.text = msg
                        }
                    }
                })
            }


            R.id.btnDataStoreGetMyData -> {
                addLog("\n\n=== DataStore 내 모든 데이터 가져오기 ===\n")
                addLog("DataStore.getMyData(listener) Called\n")

                DataStore.getMyData(object : DataStore.DataStoreMyDataListener {
                    override fun onDataStoreMyData(result: ResultAPI, myData: Map<String, String>) {
                        val msg = "DataStore.getMyData Callback\n" +
                                "\n" +
                                "$result\n" +
                                "\n" +
                                "$myData"
                        addLog(msg)

                        Handler(Looper.getMainLooper()).post {
                            tvResponse.text = msg
                        }
                    }
                })
            }


            R.id.btnDataStoreGetUsersData -> {
                addLog("\n\n=== DataStore 키로 모든 데이터 가져오기 (key 0) ===\n")
                addLog("DataStore.getUsersData(key0, listener) Called\n")

                DataStore.getUsersData(key0.text.toString(), object : DataStore.DataStoreUsersDataListener {
                    override fun onDataStoreUsersData(result: ResultAPI, key: String?, usersData: Map<Long, String>) {
                        val msg = "DataStore.getUsersData(key 0) Callback\n" +
                                "\n" +
                                "$result\n" +
                                "\n" +
                                "Key : $key\n" +
                                "\n" +
                                "$usersData"
                        addLog(msg)

                        Handler(Looper.getMainLooper()).post {
                            tvResponse.text = msg
                        }
                    }
                })
            }

        }

    }
}
